<?php
/*
Plugin Name: UMD Login
Description:  Authenticate your WordPress usernames with UMD.
Version: 2.1.1
Author: Kyle Burkhardt.
*/

//custom updates/upgrades
add_action('init', 'umd_activate_au');
function umd_activate_au()
{
    require_once('wp_auto_update.php');
    $umd_plugin_current_version = '2.1.1';
    $umd_plugin_remote_path = 'http://joomla.dts.utah.gov/wordpress/update.php';
    $umd_plugin_slug = plugin_basename(__FILE__);
    new wp_auto_update ($umd_plugin_current_version, $umd_plugin_remote_path, $umd_plugin_slug);
}

class UMDLogin
{
    static $instance = false;
    var $prefix = 'umd_';
    var $settings = array();
    var $stateRole = "state";
    var $agencyRole = "agency";

    public function __construct()
    {
        $this->settings = $this->get_settings_obj($this->prefix);

        add_action('admin_init', array($this, 'save_settings'));
        add_action('admin_menu', array($this, 'menu'));
        add_action('activated_plugin', array($this, 'save_error'));

        register_activation_hook(__FILE__, array($this, 'activate'));
        register_deactivation_hook(__FILE__, array($this, 'deactivate'));

        add_filter('authenticate', array($this, 'authenticate'), 10, 3);
        add_action('after_setup_theme', array($this, 'validateCurrentUser'), 10, 0);
    }

    public static function getInstance()
    {
        if (!self::$instance) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    function save_error()
    {
        update_option('plugin_error', ob_get_contents());

    }

    function activate()
    {

        if (array_key_exists('HTTP_UID', $_SERVER) &&
            array_key_exists('HTTP_LOGINID', $_SERVER) &&
            array_key_exists('HTTP_MAIL', $_SERVER) &&
            array_key_exists('HTTP_FIRSTNAME', $_SERVER) &&
            array_key_exists('HTTP_LASTNAME', $_SERVER) &&
            array_key_exists('HTTP_AGENCY', $_SERVER) &&
            array_key_exists('HTTP_ASSIGNEDAGENCY', $_SERVER)
        ) {

            $this->check_system_roles();

            $this->validateCurrentUser();

            $user = wp_get_current_user();

            if ($user != null && $user->ID == 2) {

                global $wpdb;
                $myrows = $wpdb->get_results("SELECT count(id) as count FROM " . $wpdb->users);
                if ($myrows[0]->count <= 2) {
                    $wpdb->get_results("update " . $wpdb->usermeta . " SET meta_value = 'a:1:{s:13:\"administrator\";b:1;}' where user_id = 2 and meta_key ='wp_capabilities';");
                }
            }

        } else {

            echo "Unable to activate, UMD headers not available<br />";

            if (!array_key_exists('HTTP_UID', $_SERVER)) {
                echo "<label style='color:red;'>UID Missing</label><br />";
            }

            if (!array_key_exists('HTTP_LOGINID', $_SERVER)) {
                echo "<label style='color:red;'>LoginId Missing</label><br />";
            }

            if (!array_key_exists('HTTP_MAIL', $_SERVER)) {
                echo "<label style='color:red;'>Email Missing</label><br />";
            }

            if (!array_key_exists('HTTP_FIRSTNAME', $_SERVER)) {
                echo "<label style='color:red;'>First Name Missing</label><br />";
            }

            if (!array_key_exists('HTTP_LASTNAME', $_SERVER)) {
                echo "<label style='color:red;'>Last Name Missing</label><br />";
            }

            if (!array_key_exists('HTTP_AGENCY', $_SERVER)) {
                echo "<label style='color:red;'>Agency Number Missing</label><br />";
            }

            if (!array_key_exists('HTTP_ASSIGNEDAGENCY', $_SERVER)) {
                echo "<label style='color:red;'>Assigned Agency Missing</label><br />";
            }

            exit;
        }


    }

    function deactivate()
    {

    }

    function check_system_roles()
    {
        // State, Agency

        $capabilities = array(
            'read' => true,  // true allows this capability
        );

        if (get_role($this->stateRole) == null) {
            $role = add_role($this->stateRole, __('State User'), $capabilities);
        }

        if (get_role($this->agencyRole) == null) {
            $role = add_role($this->agencyRole, __('Agency User'), $capabilities);
        }

    }

    function menu()
    {
        add_options_page("UMD Login", "UMD Login", 'manage_options', "umd-login", array($this, 'admin_page'));
    }

    function admin_page()
    {
        include 'UMDLoginAdmin.php';
    }

    function get_settings_obj()
    {
        return get_option("{$this->prefix}settings", false);
    }

    function set_settings_obj($newobj)
    {
        return update_option("{$this->prefix}settings", $newobj);
    }

    function set_setting($option = false, $newvalue)
    {
        if ($option === false) return false;

        $this->settings = $this->get_settings_obj($this->prefix);
        $this->settings[$option] = $newvalue;
        return $this->set_settings_obj($this->settings);
    }

    function get_setting($option, $default = false)
    {
        if (!isset($this->settings[$option])) return $default;

        return apply_filters($this->prefix . 'get_setting', $this->settings[$option], $option);
    }

    function add_setting($option = false, $newvalue)
    {
        if ($option === false) return false;

        if (!isset($this->settings[$option])) {
            return $this->set_setting($option, $newvalue);
        } else return false;
    }

    function get_field_name($setting, $type = 'string')
    {
        return "{$this->prefix}setting[$setting][$type]";
    }

    function save_settings()
    {

        if (isset($_REQUEST["{$this->prefix}setting"]) && check_admin_referer('save_umd_settings', 'save_the_umd')) {

            $new_settings = $_REQUEST["{$this->prefix}setting"];

            foreach ($new_settings as $setting_name => $setting_value) {
                foreach ($setting_value as $type => $value) {
                    if ($type == "array") {
                        $this->set_setting($setting_name, explode(";", $value));
                    } else {
                        $this->set_setting($setting_name, $value);
                    }
                }
            }
            add_action('admin_notices', array($this, 'saved_admin_notice'));
        }
    }

    function saved_new_admin()
    {
        echo '<div class="updated">
	       <p>Super admin created: ' . $_SERVER["HTTP_MAIL"] . '</p>
	    </div>';
    }

    function saved_admin_notice()
    {
        echo '<div class="updated">
	       <p>UMD Login settings have been saved.</p>
	    </div>';
    }


    function get_user_by_meta_data($meta_key, $meta_value)
    {
        umd_console("Looking up user from metadata " . $meta_key . " " . $meta_value );
        // Query for users based on the meta data
        $user_query = new WP_User_Query(
            array(
                'meta_key' => $meta_key,
                'meta_value' => $meta_value
            )
        );

        // Get the results from the query, returning the first user
        $users = $user_query->get_results();

        umd_console("Found Users: " . count($users) );

        if (empty($users)) {
            umd_console("Unable to find by " . $meta_key);
            return;
        }

        return $users[0];
    } // end get_user_by_meta_datas

    function validateCurrentUser()
    {
        umd_console("Validating user");

        // Ignore all admin-ajax requests
        $query = "/wp-admin/admin-ajax.php";
        if( substr($_SERVER["REQUEST_URI"], 0, strlen($query)) === $query ) {
                umd_console("Ignoring admin-ajax request");
                return;
        }

        if ( !isset($_SERVER["HTTP_UID"]) || $_SERVER["HTTP_UID"] == 100997576 || $_SERVER["HTTP_UID"] == 100997577) {
            umd_console("Using Anonymous");
            $user = wp_get_current_user();
            if ($user) {
                wp_destroy_current_session();
                wp_clear_auth_cookie();
            }
            return;
        }

        umd_console("UID: " . $_SERVER["HTTP_UID"]);
        $uidHeader = $_SERVER["HTTP_UID"];
        $usernameHeader = $_SERVER["HTTP_LOGINID"];
        $emailHeader = $_SERVER["HTTP_MAIL"];
        $firstNameHeader = $_SERVER["HTTP_FIRSTNAME"];
        $lastNameHeader = $_SERVER["HTTP_LASTNAME"];

        umd_console("LoginID: " . $_SERVER["HTTP_LOGINID"]);
        umd_console("Mail: " . $_SERVER["HTTP_MAIL"]);
        umd_console("FirstName: " . $_SERVER["HTTP_FIRSTNAME"]);
        umd_console("LastName: " . $_SERVER["HTTP_LASTNAME"]);

        $user = wp_get_current_user();

        if ($user->user_email === $emailHeader) {
            umd_console("Current User Matches: " . $emailHeader);
            $this->check_roles($user);

            return;
        }

        $user = $this->get_user_by_meta_data("uid", $uidHeader);

        // Try to associate existing users by email
        if ($user == null) {
            umd_console("Finding user by email: " . $emailHeader);
            $user = get_user_by('email', $emailHeader);

            if ($user == null) {
                umd_console("Finding user login: " . $usernameHeader);
                $user = get_user_by('login', $usernameHeader);
            }

            if( $user != null) {
                if (is_a($user, 'WP_User')) {
                    umd_console("Found User: " . $user->first_name);
                    add_user_meta($user->ID, 'uid', $uidHeader);
                    wp_update_user($user);
                } else {
                    umd_console("Unable to find user");
                }
            } else {
                umd_console("Unable to find user");
            }
        }

        //
        if ($user == null) {
            if ($this->get_setting("disable_auto_account_creation")) {
                umd_console("Using Anonymous");
                $user = wp_get_current_user();
                if ($user) {
                    wp_destroy_current_session();
                    wp_clear_auth_cookie();
                }
                return;
            } else {
                umd_console("Creating new user");

                $random_password = wp_generate_password($length = 12, $include_standard_special_chars = false);
                $user = wp_create_user($usernameHeader, $random_password, $emailHeader);

                umd_console(print_r($user, true));

                $user = get_user_by('email', $emailHeader);

                if (is_a($user, 'WP_User')) {

                    add_user_meta($user->ID, 'uid', $uidHeader);
                    $user->first_name = $firstNameHeader;
                    $user->last_name = $lastNameHeader;

                    wp_update_user($user);
                }
            }
        }

        // Update user info
        $updated = false;
        if ($user->uid != $uidHeader) {
            $user->uid = $uidHeader;
            $updated = true;
        }
        if ($user->user_email != $emailHeader) {
            $user->user_email = $emailHeader;
            $updated = true;
        }
        if ($user->first_name != $firstNameHeader) {
            $user->first_name = $firstNameHeader;
            $updated = true;
        }
        if ($user->last_name != $lastNameHeader) {
            $user->last_name = $lastNameHeader;
            $updated = true;
        }
        if ($user->user_login != $usernameHeader) {
            $user->user_login = $usernameHeader;
            $updated = true;
        }

        $usermeta = get_user_meta($user->ID, 'uid');
        if ($usermeta != "" && count($usermeta) != 1) {
            delete_user_meta($user, 'uid');
            add_user_meta($user->ID, 'uid', $uidHeader);
        }

        $this->check_roles($user);

        if ($updated) {
            wp_update_user($user);
        }

        wp_set_current_user($user->ID);
        wp_set_auth_cookie($user->ID, 1);

        umd_console("Authentication complete");
    }

    function check_roles($user)
    {
        umd_console("Checking Roles");
        if (array_key_exists('administrator', $user->wp_capabilities) ||
            array_key_exists('author', $user->wp_capabilities) ||
            array_key_exists('contributor', $user->wp_capabilities) ||
            array_key_exists('editor', $user->wp_capabilities) ||
            array_key_exists('super_admin', $user->wp_capabilities)
        ) {
            umd_console("Has Admin Role, ignoring");
            return;
        }

        if (!$this->get_setting("disable_auto_account_creation")) {
            if (array_key_exists('subscriber', $user->wp_capabilities) ||
                array_key_exists('agency', $user->wp_capabilities) ||
                array_key_exists('state', $user->wp_capabilities)
            ) {

                $agencyNumber = $this->get_setting("agency_number");
                $agencyHeader = $_SERVER["HTTP_AGENCY"];
                $assignedAgencyHeader = $_SERVER["HTTP_ASSIGNEDAGENCY"];

                $calculatedRole = 'subscriber';

                // Is employee
                if ($agencyHeader != null) {

                    // All emps should be state employees if the setting is checked

                    if ($this->get_setting("state_employees") == 1) {
                        $calculatedRole = $this->stateRole;
                    }


                    // Belongs to agency this site is owned by
                    if ($agencyNumber == $agencyHeader) {
                        if ($this->get_setting("agency_employees") == 1) {
                            $calculatedRole = $this->agencyRole;
                        }
                    }
                }

                /// Employee assigned to this agency
                if ($agencyHeader != null && $assignedAgencyHeader == $agencyNumber) {

                    if ($this->get_setting("agency_assigned_employees") == 1) {
                        $calculatedRole = $this->agencyRole;
                    }
                }

                // Contractor
                if ($agencyHeader == null && $assignedAgencyHeader != null) {

                    if ($this->get_setting("state_contractors") == 1) {
                        $calculatedRole = $this->stateRole;
                    }

                    if ($assignedAgencyHeader == $agencyNumber) {
                        if ($this->get_setting("agency_contractors") == 1) {
                            $calculatedRole = $this->agencyRole;
                        }
                    }
                }

                $user->set_role($calculatedRole);
                umd_console("Roles Updated");
            }

            umd_console("Roles Match");
        }
    }

    function authenticate($user, $username, $password)
    {

        // Remove default WP authentication hook
        remove_filter('authenticate', 'wp_authenticate_username_password', 20, 3);
        remove_filter('authenticate', 'wp_authenticate_cookie', 30, 3);

        $emailHeader = $_SERVER["HTTP_MAIL"];
        $uidHeader = $_SERVER["HTTP_UID"];
        $usernameHeader = $_SERVER["HTTP_LOGINID"];

        $user = $this->get_user_by_meta_data('uid', $uidHeader);

        wp_set_current_user($user->ID);
        wp_set_auth_cookie($user->ID, 1);

        return new WP_User($user);
    }


}

if (!function_exists('auth_redirect')) {

    function auth_redirect()
    {

        UMDLogin::getInstance()->validateCurrentUser();
    }
}

if (!function_exists('umd_console')) {

    function umd_console($data)
    {
        if (isset($_GET['debug']) && $_GET['debug']) {

            if (is_array($data))
                $output = "<script>console.log('" . strip_tags(implode(',', $data)) . "');</script>";
            else
                $output = "<script>console.log('" . strip_tags($data) . "');</script>";

            echo $output;
        }
    }
}

if (!function_exists('umd_strleft')) {
    function umd_strleft($s1, $s2)
    {
        return substr($s1, 0, strpos($s1, $s2));
    }
}
if (!function_exists('umd_selfUrl')) {
    function umd_selfUrl()
    {
        // Returns the url of the current page
        if (!isset($_SERVER['REQUEST_URI'])) {
            $serverrequri = $_SERVER['PHP_SELF'];
        } else {
            $serverrequri = $_SERVER['REQUEST_URI'];
        }
        $s = empty($_SERVER["HTTPS"]) ? '' : ($_SERVER["HTTPS"] == "on") ? "s" : "";
        $protocol = umd_strleft(strtolower($_SERVER["SERVER_PROTOCOL"]), "/") . $s;
        $port = ($_SERVER["SERVER_PORT"] == "80") ? "" : (":" . $_SERVER["SERVER_PORT"]);
        return $protocol . "://" . $_SERVER['SERVER_NAME'] . $port . $serverrequri;
    }
}

function umd_logout_without_confirm($action, $result)
{
    /**
     * Allow logout without confirmation
     */
    if ($action == "log-out" && !isset($_GET['_wpnonce'])) {
        $redirect_to = isset($_REQUEST['redirect_to']) ? $_REQUEST['redirect_to'] : '';
        $location = str_replace('&amp;', '&', wp_logout_url($redirect_to));;
        header("Location: $location");
        die;
    }
}
add_action('check_admin_referer', 'umd_logout_without_confirm', 10, 2);

function umd_sso_logout()
{
    if (!$_SERVER['HTTP_REFERER']) {
        $redirect_to = UMDLogin::getInstance()->get_setting('logout_url', 'https://id.utah.gov/logout') . "?" . UMDLogin::getInstance()->get_setting('goto_param', 'goto') ."=" . umd_selfUrl();
    } else {
        $redirect_to = UMDLogin::getInstance()->get_setting('logout_url', 'https://id.utah.gov/logout') . "?" . UMDLogin::getInstance()->get_setting('goto_param', 'goto') ."=" . $_SERVER['HTTP_REFERER'];
    }

    wp_redirect(wp_nonce_url($redirect_to));

    exit();
}

add_action('wp_logout', 'umd_sso_logout');

if (!function_exists('umd_my_update_cookie')) {
    function umd_my_update_cookie($logged_in_cookie)
    {
        $_COOKIE[LOGGED_IN_COOKIE] = $logged_in_cookie;
    }
}
add_action( 'set_logged_in_cookie', 'umd_my_update_cookie' );

if (!function_exists('umd_str_true')) {
    /**
     * Evaluates natural language strings to boolean equivalent
     *
     * Used primarily for handling boolean text provided in shopp() tag options.
     * All values defined as true will return true, anything else is false.
     *
     * Boolean values will be passed through.
     *
     * Replaces the 1.0-1.1 value_is_true()
     *
     * @author Jonathan Davis
     * @since 1.2
     *
     * @param string $string The natural language value
     * @param array $istrue A list strings that are true
     * @return boolean The boolean value of the provided text
     **/
    function umd_str_true($string, $istrue = array('yes', 'y', 'true', '1', 'on', 'open'))
    {
        if (is_array($string)) return false;
        if (is_bool($string)) return $string;
        return in_array(strtolower($string), $istrue);
    }
}




$UMDLogin = UMDLogin::getInstance();

